#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// This indicator will automatically draw fibonacci retracements from swing highs and
	/// lows in the market.
	/// 
	/// Code Written by Tyler Moore
	/// www.FreeIndicators.Com
	/// 
	/// For modifications to this code or to have your own custom code written
	/// please visit www.MooreTechLLC.com	
    /// </summary>
    [Description("Enter the description of your new custom indicator here")]
    public class AutoFibRetracements : Indicator
    {
        #region Variables
        // Wizard generated variables
            private int strength = 20; // Default setting for Strength
        // User defined variables (add any user defined variables below)
		private IFibonacciRetracements fib;
		private MIN min;
		private MAX max;
		private int barsSinceHigh = -1, barsSinceLow = -1;		
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            Overlay				= true;
        }
		
		protected override void OnStartUp()
		{
			min=MIN(Low,strength);
			max=MAX(High,strength);
		}

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			if (CurrentBar<strength)
				return;
			
			if (High[0]>max[1])
				barsSinceHigh=0;
			else if (barsSinceHigh>-1)
				barsSinceHigh++;
			
			if (Low[0]<min[1])
				barsSinceLow=0;
			else if (barsSinceLow>-1)
				barsSinceLow++;
			
			if (Math.Max(barsSinceHigh,barsSinceLow)<=0)
				return;
			
			if (barsSinceHigh<barsSinceLow)			
				fib=DrawFibonacciRetracements("fib",AutoScale,barsSinceLow,Low[barsSinceLow],barsSinceHigh,High[barsSinceHigh]);			
			else
				fib=DrawFibonacciRetracements("fib",AutoScale,barsSinceHigh,High[barsSinceHigh],barsSinceLow,Low[barsSinceLow]);
        }

        #region Properties

        [Description("")]
        [GridCategory("Parameters")]
        public int Strength
        {
            get { return strength; }
            set { strength = Math.Max(1, value); }
        }
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private AutoFibRetracements[] cacheAutoFibRetracements = null;

        private static AutoFibRetracements checkAutoFibRetracements = new AutoFibRetracements();

        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        public AutoFibRetracements AutoFibRetracements(int strength)
        {
            return AutoFibRetracements(Input, strength);
        }

        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        public AutoFibRetracements AutoFibRetracements(Data.IDataSeries input, int strength)
        {
            if (cacheAutoFibRetracements != null)
                for (int idx = 0; idx < cacheAutoFibRetracements.Length; idx++)
                    if (cacheAutoFibRetracements[idx].Strength == strength && cacheAutoFibRetracements[idx].EqualsInput(input))
                        return cacheAutoFibRetracements[idx];

            lock (checkAutoFibRetracements)
            {
                checkAutoFibRetracements.Strength = strength;
                strength = checkAutoFibRetracements.Strength;

                if (cacheAutoFibRetracements != null)
                    for (int idx = 0; idx < cacheAutoFibRetracements.Length; idx++)
                        if (cacheAutoFibRetracements[idx].Strength == strength && cacheAutoFibRetracements[idx].EqualsInput(input))
                            return cacheAutoFibRetracements[idx];

                AutoFibRetracements indicator = new AutoFibRetracements();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.Strength = strength;
                Indicators.Add(indicator);
                indicator.SetUp();

                AutoFibRetracements[] tmp = new AutoFibRetracements[cacheAutoFibRetracements == null ? 1 : cacheAutoFibRetracements.Length + 1];
                if (cacheAutoFibRetracements != null)
                    cacheAutoFibRetracements.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheAutoFibRetracements = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.AutoFibRetracements AutoFibRetracements(int strength)
        {
            return _indicator.AutoFibRetracements(Input, strength);
        }

        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        public Indicator.AutoFibRetracements AutoFibRetracements(Data.IDataSeries input, int strength)
        {
            return _indicator.AutoFibRetracements(input, strength);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.AutoFibRetracements AutoFibRetracements(int strength)
        {
            return _indicator.AutoFibRetracements(Input, strength);
        }

        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        public Indicator.AutoFibRetracements AutoFibRetracements(Data.IDataSeries input, int strength)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.AutoFibRetracements(input, strength);
        }
    }
}
#endregion
